import time
from pathlib import Path
import requests
import json
import re
import os
from datetime import datetime
from docx import Document
from docx.shared import RGBColor, Pt, Cm
from docx.oxml.ns import qn
from docx.oxml.shared import OxmlElement
from docx.enum.text import WD_ALIGN_PARAGRAPH
from PIL import Image
import logging

session = requests.Session()


def create_path(path: str) -> None:
    if not os.path.exists(path):
        os.makedirs(path)


def get_article_html(cookie: str, article_id: str):
    """
    get article html
    :param cookie: website's cookie information
    :param article_id: article's url such as 1097430290934005800 (https://www.bilibili.com/opus/1097430290934005800?spm_id_from=333.1387.0.0)
    :return: html
    """
    url = f"https://www.bilibili.com/opus/{article_id}"
    headers = {
        'cookie': cookie,
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36'
    }
    response = session.get(url, headers=headers)
    return response.text


def get_p_tag_text(html) -> tuple:
    result = html.xpath('./span | ./strong')
    color = result[0].get('style')
    if color is not None:
        color = re.findall(r'color:(.*?);', result[0].get('style'))[0]
    is_strong = result[0].tag == 'strong'
    if len(result) != 0:
        return ''.join(result[i].text for i in range(0, len(result))), color, is_strong
    else:
        return '\n', color, is_strong


def get_img(cookie, article_id: str, url_list: list, path: Path) -> list:
    """
    get img.
    :param cookie: website's cookie information
    :param article_id: article's ID
    :param url_list: each item under the opus-module-content class
    :param path: img path
    :return: image path list
    """
    result = list()

    headers = {
        'cookie': cookie,
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36'
    }
    for url in url_list:
        response = session.get(url, headers=headers)
        time.sleep(0.5)

        name = url.split('/')[-1]

        Path(path/article_id).mkdir(parents=True, exist_ok=True)

        with open(path/article_id/name, 'wb') as f:
            f.write(response.content)

        logging.log(logging.INFO,'get' + str(path/article_id/name))

        result.append(path/article_id/name)

    return result


def get_rgb(node: dict) -> tuple:
    """
    get rgb color
    :param node: node
    :return: r, g, b
    """
    if 'color' in node['word'].keys() and node['word']['color'] is not None:
        color = node['word']['color'].strip('#')
        return int(color[0:2], 16), int(color[2:4], 16), int(color[4:6], 16)
    else:
        return 47, 50, 56


def add_hyperlink(paragraph, text, url, color="00699D", underline=True):
    """
    add hyperlink
    :param paragraph: paragraph
    :param text: text to be added
    :param url: link url
    :param color: color for hyperlink,default 00699D
    :param underline: text underline for hyperlink,default True
    :return: underline element
    """
    # get document part
    part = paragraph.part
    # create a relationship
    r_id = part.relate_to(url, "http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink",
                          is_external=True)

    # create hyperlink element
    hyperlink = OxmlElement('w:hyperlink')
    hyperlink.set(qn('r:id'), r_id)

    # create text run
    new_run = OxmlElement('w:r')

    # create runtime attributes
    rPr = OxmlElement('w:rPr')

    # set color
    color_elem = OxmlElement('w:color')
    color_elem.set(qn('w:val'), color)
    rPr.append(color_elem)

    # set underline
    if underline:
        u_elem = OxmlElement('w:u')
        u_elem.set(qn('w:val'), 'single')  # 单下划线
        rPr.append(u_elem)

    new_run.append(rPr)

    # create text element
    text_elem = OxmlElement('w:t')
    text_elem.text = text
    text_elem.set(qn('xml:space'), 'preserve')

    new_run.append(text_elem)

    # add run to hyperlink
    hyperlink.append(new_run)

    paragraph._p.append(hyperlink)

    return hyperlink


def add_text(doc, item: dict, align: int = 0) -> None:
    """
    Add text to document.
    :param doc: docx document
    :param item: item's text
    :return: None
    """
    paragraph = doc.paragraphs
    is_first = True
    for node in item:
        if node['type'] == 'TEXT_NODE_TYPE_WORD':
            # font size
            fontSize = node['word']['font_size']

            # get bold
            if 'bold' in node['word']['style'].keys():
                is_blod = node['word']['style']['bold']
            else:
                is_blod = False

            words = node['word']['words']

            # get color
            r, g, b = get_rgb(node)

            if len(item) > 1 and is_first is False:
                run = paragraph.add_run(words)
                run.font.color.rgb = RGBColor(r, g, b)
                run.font.size = Pt(fontSize / 1.5)
                run.font.name = "黑体"
                run._element.rPr.rFonts.set(qn('w:eastAsia'), '黑体')
                run.bold = is_blod

            else:
                paragraph = doc.add_paragraph(words)
                for run in paragraph.runs:
                    run.font.color.rgb = RGBColor(r, g, b)
                    run.font.size = Pt(fontSize / 1.5)
                    run.font.name = "黑体"
                    run._element.rPr.rFonts.set(qn('w:eastAsia'), '黑体')
                    run.bold = is_blod

            is_first = False
        elif node['type'] == 'TEXT_NODE_TYPE_RICH':
            text = node['rich']['text']
            if 'jump_url' in node['rich'].keys():
                url = 'https://' + node['rich']['jump_url'].strip('http:').strip('https:').strip('//')
                add_hyperlink(paragraph, text, url)
            else:
                run = paragraph.add_run(text)
                run.font.color.rgb = RGBColor(47, 50, 56)
                run.font.size = Pt(17 / 1.5)
                run.font.name = "黑体"
                run._element.rPr.rFonts.set(qn('w:eastAsia'), '黑体')

    if align == 1:
        paragraph.alignment = WD_ALIGN_PARAGRAPH.CENTER


def add_image(doc, image_path_list) -> None:
    """
    Add img to document.
    :param doc: docx document
    :param image_path_list: image path list
    :return: None
    """
    section = doc.sections[0]

    available_width = (section.page_width - section.left_margin - section.right_margin)

    for image_path in image_path_list:
        img = Image.open(image_path)
        img.save(image_path)
        img.close()
        doc.add_picture(str(image_path), width=available_width)


def get_module(INITIAL_STATE: dict, module_name: str):
    INITIAL_STATE = INITIAL_STATE['detail']['modules']
    for item in INITIAL_STATE:
        if module_name in ''.join(list(item.keys())):
            return item
    return None


def add_auto_numbered_data(doc, items: list) -> None:
    """
    add auto numbered data.
    :param doc: docx document.
    :param items: items list
    :return: None
    """
    for nodes in items:
        order = nodes['order']
        nodes['nodes'][0]['word']['words'] = str(order) + '. ' + nodes['nodes'][0]['word']['words']
        add_text(doc, nodes['nodes'])


def add_code_text(doc, item: dict) -> None:
    content = item['code']['content']
    paragraph = doc.add_paragraph(content)
    for run in paragraph.runs:
        run.font.name = 'Courier New'
        run.font.size = Pt(14 / 2)
        run.font.color.rgb = RGBColor(47, 50, 56)


def add_link_card(doc, item: dict) -> None:
    """
    add link card.
    :param doc: docx document.
    :param item: items list
    :return: None
    """
    paragraph = doc.add_paragraph('\n')
    if 'opus' in item['link_card']['card']:
        title_and_url = item['link_card']['card']['opus']
    elif 'ugc' in item['link_card']['card']:
        title_and_url = item['link_card']['card']['ugc']

    add_hyperlink(paragraph, title_and_url['title'], title_and_url['jump_url'])
    doc.add_paragraph('\n')


def get_article(cookie: str, article_id: str, doc_storage_location: str = None, document_name: str = 'Document.doc',
                img_path: str = None) -> str:
    """
    get_article
    :param cookie: website's cookie information.
    :param article_id: article's url such as 1097430290934005800 (https://www.bilibili.com/opus/1097430290934005800?spm_id_from=333.1387.0.0)
    :param doc_storage_location: doc local storage in word.
    :param document_name: document name.
    :param img_path: img path
    :return: document text
    """
    if doc_storage_location is None:
        doc_storage_location = Path.cwd()
    else:
        doc_storage_location = Path(Path.cwd() / doc_storage_location)

    if img_path is None:
        img_path = Path.cwd() / "img"
    else:
        img_path = Path(img_path)

    html = get_article_html(cookie, article_id)
    INITIAL_STATE = re.findall(r'window.__INITIAL_STATE__=(.*);\(function', html)[0]
    INITIAL_STATE = json.loads(INITIAL_STATE)

    # with open('test.json', 'w', encoding='utf-8') as f:
    #     json.dump(INITIAL_STATE, f, ensure_ascii=False)

    title = get_module(INITIAL_STATE, 'title')
    author = get_module(INITIAL_STATE, 'author')
    module_top = get_module(INITIAL_STATE, 'module_top')
    module_content = get_module(INITIAL_STATE, 'module_content')

    doc = Document()
    section = doc.sections[0]
    section.page_width = Cm(21)
    section.page_height = Cm(29.7)

    if title is not None:
        title = title['module_title']['text']
        doc_title = doc.add_heading(title, level=1)
        for run in doc_title.runs:
            run.font.color.rgb = RGBColor(0, 0, 0)
            run.font.size = Pt(22 / 1.5)

    if author is not None:
        author_name = author['module_author']['name']
        author_time = author['module_author']['pub_time']

        if '编辑于' in author_time:
            author_time = datetime.strptime(author_time, '编辑于 %Y年%m月%d日 %H:%M')
        else:
            author_time = datetime.strptime(author_time, '%Y年%m月%d日 %H:%M')
    else:
        author_name = None
        author_time = None

    if author_name is not None and author_time is not None:
        doc_name = doc.add_paragraph(author_name)
        doc_time = doc.add_paragraph(author_time.strftime("%Y/%m/%d %H:%M"))

        for run in doc_name.runs:
            run.font.color.rgb = RGBColor(0, 0, 0)
            run.font.size = Pt(17 / 1.5)
            run.font.name = "黑体"
            run._element.rPr.rFonts.set(qn('w:eastAsia'), '黑体')

        for run in doc_time.runs:
            run.font.color.rgb = RGBColor(148, 153, 160)
            run.font.size = Pt(13 / 1.5)

    top_pic_urls = list()
    if module_top is not None:
        module_tops = module_top['module_top']['display']['album']['pics']
        for pic in module_tops:
            top_pic_urls.append(pic['url'])

    add_image(doc, get_img(cookie, article_id, top_pic_urls, img_path))

    if module_content is not None:
        module_content = module_content['module_content']['paragraphs']

    for item in module_content:
        if item['para_type'] == 1:
            add_text(doc, item['text']['nodes'], item['align'])
        elif item['para_type'] == 2:
            pic_urls = list()
            pics = item['pic']['pics']
            for pic in pics:
                pic_urls.append(pic['url'])
            add_image(doc, get_img(cookie, article_id, pic_urls, img_path))
            logging.log(logging.INFO, 'add picture urls: {}'.format(pic_urls))
        elif item['para_type'] == 3:
            logging.log(logging.WARNING,'I don\'t know this para_type. If possible, please send this article to my GitHub ISSUES.(https://github.com/Guoziqi329/bilibili_crawling/issues)')
        elif item['para_type'] == 4:
            logging.log(logging.WARNING,'I don\'t know this para_type. If possible, please send this article to my GitHub ISSUES.(https://github.com/Guoziqi329/bilibili_crawling/issues)')
        elif item['para_type'] == 5:
            add_auto_numbered_data(doc, item['list']['items'])
            logging.log(logging.INFO, 'add auto_numbered_data')
        elif item['para_type'] == 6:
            add_link_card(doc, item)
            logging.log(logging.INFO,'add link_card')
        elif item['para_type'] == 7:
            add_code_text(doc, item)
            logging.log(logging.INFO,'add code_text')

    doc_storage_location.mkdir(parents=True, exist_ok=True)
    doc.save(str(doc_storage_location / document_name))

    logging.log(logging.INFO,f'{title}--{author_name}--{author_time} finished.')

    return "\n".join([s.text for s in doc.paragraphs])


from src.bili_scraper.bili_scraper import BiliScraper
import json

if __name__ == '__main__':
    with open("cookie.json", "r", encoding="utf-8") as f:
        cookie = json.load(f)['cookie']

    bilibili = BiliScraper(cookie)

    # bilibili.getVideo('BV1Jd1oB3EGD','video/video')

    comments = bilibili.getVideoComments('BV1om2FBVEhS', 'img', 1)

    with open("comments.json", "w", encoding="utf-8") as f:
        json.dump(comments, f, ensure_ascii=False)

    # dm = bilibili.getVideoDm("BV1YA4y1R7RJ")
    # with open('dm.json', 'w', encoding='utf-8') as f:
    #     json.dump(dm, f, ensure_ascii=False)

    # bilibili.getArticle("1102463389176692741", 'doc', 'doc.docx', 'img/test/')


import os
import time
import requests
import json
import re
import tempfile
from moviepy.video.io.VideoFileClip import VideoFileClip
from moviepy.audio.io.AudioFileClip import AudioFileClip
import logging
from pathlib import Path

session = requests.Session()


def get_html(cookie: str, url: str) -> str:
    """
    get web page
    :param cookie: website's cookie information
    :param url: website's url
    :return: page html
    """
    headers = {
        'cookie': cookie,
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36'
    }
    response = session.get(url, headers=headers)
    return response.text


def get_playinfo(html: str) -> dict:
    """
    get play information
    :param html: web page html
    :return: play information
    """
    info = re.findall('window.__playinfo__=(.*?)</script>', html)[0]
    info = json.loads(info)
    return info


def get_video_AND_audio_url(playinfo: dict, video_quality) -> tuple:
    """
    get video url and audio url
    :param playinfo: play information
    :param video_quality: video quality
    :return: video url and audio url
    """
    return playinfo['data']['dash']['video'][(video_quality - 1) * 3]['baseUrl'], playinfo['data']['dash']['audio'][0][
        'baseUrl']


def get_video(cookie: str, video_id: str, output_dir: str = None, select_video_quality: bool = False) -> None:
    """
    get video
    :param cookie: website's cookie information
    :type cookie: str
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :type video_id: str
    :param output_dir: the folder where the video will be saved
    :type output_dir: str
    :param select_video_quality: whether to choose video quality, default is not selected, video quality is the highest.
    :type select_video_quality: bool
    :return: None
    """
    if output_dir is None:
        output_dir = Path.cwd()
    else:
        output_dir = Path(output_dir)

    output_dir.mkdir(parents=True, exist_ok=True)

    url = f'https://www.bilibili.com/video/{video_id}/'

    html = get_html(cookie, url)

    playinfo = get_playinfo(html)

    accept_description = playinfo['data']['accept_description']
    accept_quality = playinfo['data']['accept_quality']

    if select_video_quality:
        video_quality = -1
        while video_quality not in range(1, len(accept_quality) + 1):
            print("please select video quality")
            for i in range(1, len(accept_quality) + 1):
                print(f'{i} : {accept_description[i - 1]}', end='\t')
            video_quality = int(input('\n'))
            if video_quality not in range(1, len(accept_quality) + 1):
                logging.error('Parameter error, please select again.')
    else:
        video_quality = 1

    video_url, audio_url = get_video_AND_audio_url(playinfo, video_quality)

    headers = {
        'Referer': url,
        'cookie': cookie,
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36'
    }

    video_content = requests.get(url=video_url, headers=headers).content
    audio_content = requests.get(url=audio_url, headers=headers).content

    with tempfile.NamedTemporaryFile(delete=False, suffix=".mp4") as tmp_video:
        tmp_video.write(video_content)
        video_path = tmp_video.name

    with tempfile.NamedTemporaryFile(delete=False, suffix=".mp3") as tmp_audio:
        tmp_audio.write(audio_content)
        audio_path = tmp_audio.name

    try:
        video_clip = VideoFileClip(video_path)
        audio_clip = AudioFileClip(audio_path)

        final_clip = video_clip.with_audio(audio_clip)

        final_clip.write_videofile(output_dir / 'output.mp4')
        time.sleep(1)

    finally:
        try:
            os.remove(video_path)
            os.remove(audio_path)
        except Exception:
            pass


import os
import requests
import logging
import time
import re
from pathlib import Path
from .get_w_rid_And_wts import get_wbiImgKey_and_wbiSubKey, get_w_rid_And_wts, encodeURIComponent

session = requests.Session()


def get_oid(cookie: str, video_id: str) -> str:
    """
    get video oid by video id
    :param cookie: website's cookie information
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :return: video's oid
    """
    url = f"https://www.bilibili.com/video/{video_id}/"
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
        "cookie": cookie,
    }
    response = session.get(url, headers=headers)
    return re.search(r'"aid":\d+', response.text)[0][6:]


def get_offset(response) -> str:
    """
    get next offset
    :param response: requests response
    :return: next offset (str)
    """
    return response.json()['data']['cursor']['pagination_reply']['next_offset']


def get_jump_url_list(item: dict) -> list:
    """
    get jump url list
    :param item: replies item
    :return: list
    """
    jump_url_list = list()

    if len(item['content']['jump_url'].keys()) > 0:
        for key in item['content']['jump_url'].keys():
            if 'pc_url' in item['content']['jump_url'][key]:
                jump_url_list.append(item['content']['jump_url'][key]['pc_url'])
            else:
                jump_url_list.append(key if '/' in key else f'https://b23.tv/{key}')

    return jump_url_list


def get_comments_on_the_comment(cookie: str, oid: str, root: str, pages: int, delay: int = 3) -> list:
    """
    get comments on a comment
    :param cookie: website's cookie information
    :param oid: video id
    :param root: the rpid_str of the upper level comment
    :param pages: the number of pages in the reply
    :param delay: Interval time for initiating requests, the default value is 3.
    :return: second-level comment information
    """
    url = 'https://api.bilibili.com/x/v2/reply/reply'
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
        "cookie": cookie,
    }

    comments = list()

    for i in range(1, pages + 1):
        payload = {
            "oid": oid,
            "type": "1",
            "root": root,
            "ps": "10",
            "pn": str(i),
            "web_location": "333.788"
        }
        response = session.get(url, headers=headers, params=payload)
        for item in response.json()['data']['replies']:
            if "at_name_to_mid_str" in item['content'].keys():
                at_information = {"at_name": list(item['content']['at_name_to_mid_str'].keys())[0],
                                  "pid": list(item['content']['at_name_to_mid_str'].values())[0]}
            else:
                at_information = None

            jump_url_list = get_jump_url_list(item)

            comments.append({"rpid": item['rpid_str'],
                             "message": item['content']['message'],
                             "at_information": at_information,
                             "jump_url": jump_url_list})
        time.sleep(delay)

    return comments


def create_path(path: str) -> None:
    if not os.path.exists(path):
        os.makedirs(path)


def get_img(url: str, img_dir: Path, name: str) -> Path:
    """
    get img from url
    :param url: img url
    :param img_dir: directory of images
    :param name: img name
    :return: img path
    """
    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36"
    }
    response = session.get(url, headers=headers)

    img_dir.mkdir(parents=True, exist_ok=True)

    with open(img_dir / f'{name}.{url.split('.')[-1]}', 'wb') as f:
        f.write(response.content)
    return Path.cwd() / img_dir / f'{name}.{url.split('.')[-1]}'


def get_img_list(item: dict, delay: int, img_dir: Path, video_id: str) -> list:
    """
    get img list
    :param item: replies item
    :param delay: Interval time for initiating requests
    :param img_dir: directory of images
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :return: list
    """
    img_list = list()

    i = 1
    if 'pictures' in item['content'].keys():
        for pic in item['content']['pictures']:
            img_list.append({"img_src": pic["img_src"],
                             "img_path": str(
                                 get_img(pic['img_src'], img_dir / video_id / item["rpid_str"], str(i)))})
            i += 1
            time.sleep(delay)

    return img_list


def process_response(cookie, response, comments: list, oid: str, video_id: str, img_dir: str = None,
                     delay: int = 3) -> list:
    """
    Response handling
    :param cookie: website's cookie information
    :param response: response from bilibili
    :param comments: comments list
    :param oid: video id
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :param img_dir: directory of images
    :param delay: Interval time for initiating requests, the default value is 3.
    :return: comments list
    """
    if img_dir is None:
        img_dir = Path.cwd()
    else:
        img_dir = Path(img_dir)

    # get top replies
    if (top_replies := response.json()['data']['top_replies']) is not None and len(top_replies) > 0:
        second_level_comments = get_comments_on_the_comment(cookie, oid, top_replies[0]['rpid_str'],
                                                            int(top_replies[0]['rcount'] / 10) + 1
                                                            if top_replies[0]['rcount'] % 10 != 0
                                                            else int(top_replies[0]['rcount'] / 10), delay)

        img_list = get_img_list(top_replies[0], delay, img_dir, video_id)

        jump_url_list = get_jump_url_list(top_replies[0])

        comments.append({'rpid': top_replies[0]['rpid_str'], 'message': top_replies[0]['content']['message'],
                         'reply': second_level_comments, 'img': img_list, 'jump_url': jump_url_list})

        logging.log(logging.INFO, f'get top replies {top_replies[0]['rpid_str']}')

    for item in response.json()['data']['replies']:
        if item['rcount'] > 0:
            second_level_comments = get_comments_on_the_comment(cookie, oid, item['rpid_str'],
                                                                int(item['rcount'] / 10) + 1
                                                                if item['rcount'] % 10 != 0
                                                                else int(item['rcount'] / 10), delay)
        else:
            second_level_comments = None

        img_list = get_img_list(item, delay, img_dir, video_id)

        jump_url_list = get_jump_url_list(item)

        comments.append({'rpid': item['rpid_str'], 'message': item['content']['message'],
                         'reply': second_level_comments, 'img': img_list, 'jump_url': jump_url_list})

        logging.log(logging.INFO, f'get replies {item['rpid_str']}')

    return comments


def get_video_comments(cookie: str, video_id: str, img_path: str = None, delay: int = 3) -> list:
    """
    get video comments
    :param cookie: website's cookie information
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :param img_path: directory of images
    :param delay: Interval time for initiating requests, the default value is 3.
    :return: comment information
    """

    comments = list()

    oid = get_oid(cookie, video_id)

    payload = {
        "oid": oid,
        "type": "1",
        "mode": "3",
        "pagination_str": "{\"offset\":\"\"}",
        "plat": "1",
        "seek_rpid": None,
        "web_location": "1315875"
    }

    wbiImgKey, wbiSubKey = get_wbiImgKey_and_wbiSubKey(cookie)
    w_rid, wts = get_w_rid_And_wts(wbiImgKey, wbiSubKey, payload)
    payload["w_rid"] = w_rid
    payload["wts"] = wts

    headers = {
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
        "cookie": cookie,
    }

    response = session.get("https://api.bilibili.com/x/v2/reply/wbi/main", headers=headers, params=payload)

    comments = process_response(cookie, response, comments, oid, video_id, img_path, delay)

    time.sleep(delay)

    while response.json()['data']['cursor']['pagination_reply'] is not None:
        next_offset = get_offset(response)
        payload_2 = {
            "oid": oid,
            "type": "1",
            "mode": "3",
            "pagination_str": "{\"offset\":\"" + next_offset + "\"}",
            "plat": "1",
            "web_location": "1315875"
        }

        wbiImgKey, wbiSubKey = get_wbiImgKey_and_wbiSubKey(cookie)
        w_rid, wts = get_w_rid_And_wts(wbiImgKey, wbiSubKey, payload_2)
        payload_2["w_rid"] = w_rid
        payload_2["wts"] = wts
        response = session.get("https://api.bilibili.com/x/v2/reply/wbi/main", headers=headers, params=payload_2)

        # Crawling finished, exit the loop.
        if len(response.json()['data']['replies']) == 0:
            break

        comments = process_response(cookie, response, comments, oid, video_id, img_path, delay)

        time.sleep(delay)

    return comments



from .api_reverse import get_article
from .api_reverse.video import get_video ,get_video_dm, get_video_comments


class BiliScraper:
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self, cookie):
        self.cookie = cookie

    def setCookie(self, cookie):
        self.cookie = cookie

    def getCookie(self):
        return self.cookie

    def getArticle(self, article_id, doc_storage_location = None, document_name = 'Document.doc', img_path = None):
        return get_article(self.cookie, article_id, doc_storage_location, document_name, img_path)

    def getVideo(self, video_id, path = None, select_video_quality = False):
        get_video(self.cookie, video_id, path, select_video_quality)

    def getVideoDm(self, video_id):
        return get_video_dm(self.cookie, video_id)

    def getVideoComments(self, video_id, img_path = None, delay = 3):
        return get_video_comments(self.cookie, video_id, img_path, delay)





import requests
import json
import re
from .get_w_rid_And_wts import get_w_rid_And_wts, get_wbiImgKey_and_wbiSubKey
from google.protobuf.json_format import MessageToJson

# dm_pb2 使用了 SocialSisterYi (https://github.com/SocialSisterYi) 的代码(https://github.com/SocialSisterYi/bilibili-API-collect/blob/master/grpc_api/bilibili/community/service/dm/v1/dm.proto)
# 许可协议: CC-BY-NC 4.0 (https://creativecommons.org/licenses/by-nc/4.0/)
from . import dm_pb2
from .get_video import get_playinfo, get_html

session = requests.Session()

my_seg = dm_pb2.DmSegMobileReply()


def get__INITIAL_STATE__(html: str) -> dict:
    """
    get __INITIAL_STATE__
    :param html: web page html
    :return: web __INITIAL_STATE__
    """
    INITIAL_STATE = re.findall(r'window.__INITIAL_STATE__=(.*);\(function', html)[0]
    INITIAL_STATE = json.loads(INITIAL_STATE)
    return INITIAL_STATE


def get_pid(cookie: str, video_id: str) -> str:
    """
    get video pid by video id
    :param cookie: website's cookie information
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :return: video's pid
    """
    url = f"https://www.bilibili.com/video/{video_id}/"

    html = get_html(cookie, url)

    INITIAL_STATE = get__INITIAL_STATE__(html)

    return str(INITIAL_STATE["aid"])


def get_oid(cookie: str, video_id: str) -> str:
    """
    get video oid by video id
    :param cookie: website's cookie information
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :return: video's oid
    """
    url = f"https://www.bilibili.com/video/{video_id}/"

    html = get_html(cookie, url)

    playinfo = get_playinfo(html)

    return str(playinfo["data"]["last_play_cid"])


def get_video_dm(cookie: str, video_id: str) -> list:
    """
    Get video dm
    :param cookie: website's cookie information
    :param video_id: the id in the url, such as BV1Mg8RzFExV
    :return: dm list
    """
    headers = {
        "cookie": cookie,
        "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
    }

    oid = get_oid(cookie, video_id)
    pid = get_pid(cookie, video_id)

    dm_list = list()

    i = 0

    ps = [0, 120000]
    pe = [120000, 360000]

    url = 'https://api.bilibili.com/x/v2/dm/wbi/web/seg.so'

    while True:
        if i < 2:
            payload = {
                "type": "1",
                "oid": oid,
                "pid": pid,
                "segment_index": "1",
                "pull_mode": "1",
                "ps": ps[i],
                "pe": pe[i],
                "web_location": "1315873"
            }
        else:
            payload = {
                "type": "1",
                "oid": oid,
                "pid": pid,
                "segment_index": str(i),
                "web_location": "1315873"
            }

        wbiImgKey, wbiSubKey = get_wbiImgKey_and_wbiSubKey(cookie)

        w_rid, wts = get_w_rid_And_wts(wbiImgKey, wbiSubKey, payload)

        payload["w_rid"] = w_rid
        payload["wts"] = wts

        response = session.get(url, headers=headers, params=payload)
        my_seg.ParseFromString(response.content)

        if len(my_seg.elems) == 0:
            break

        for item in json.loads(MessageToJson(my_seg))['elems']:
            dm_list.append({'id': item['id'] if 'id' in item.keys() else None,
                            'color': item['color'] if 'color' in item.keys() else None,
                            'ctime': item['ctime'] if 'ctime' in item.keys() else None,
                            'progress': item['progress'] if 'progress' in item.keys() else -1,
                            'content': item['content'] if 'content' in item.keys() else None})

        i = i + 1

    return dm_list


import requests
import time
import urllib.parse
from hashlib import md5


def get_wbiImgKey_and_wbiSubKey(cookie: str):
    """
    get wbiImgKey and wbiSubKey
    :param cookie: website's cookie information
    :return: wbiImgKey, wbiSubKey
    """
    url = 'https://api.bilibili.com/x/web-interface/nav'
    headers = {
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36',
        'cookie': cookie
    }
    response = requests.get(url, headers=headers)
    wbi_img = response.json()["data"]["wbi_img"]
    return wbi_img["img_url"].split('/')[-1].split('.')[0], wbi_img["sub_url"].split('/')[-1].split('.')[0]


def encodeURIComponent(s: str) -> str:
    """
    encode url component
    :param s: str
    :return: str
    """
    return urllib.parse.quote(s, safe='')


def get_w_rid_And_wts(wbiImgKey: str, wbiSubKey: str, e: dict):
    """
    get w_rid and wts
    :param wbiImgKey: wbiImgKey
    :param wbiSubKey: wbiSubKey
    :param e: requests payload, each key must be a string and each value must be a string or None
    :return: w_rid, wts
    """
    t = wbiImgKey + wbiSubKey
    r = list()
    MIXIN_KEY_ENC_TAB = [46, 47, 18, 2, 53, 8, 23, 32, 15, 50, 10, 31, 58, 3, 45, 35, 27, 43, 5, 49, 33, 9, 42, 19, 29,
                         28, 14, 39, 12, 38, 41, 13, 37, 48, 7, 16, 24, 55, 40, 61, 26, 17, 0, 1, 60, 51, 30, 4, 22, 25,
                         54, 21, 56, 59, 6, 63, 57, 62, 11, 36, 20, 34, 44, 52]
    for i in MIXIN_KEY_ENC_TAB:
        if i < len(t):
            r.append(t[i])
    a = ''.join(r)[0:32]

    u = round(time.time())
    e["wts"] = str(u)

    c = sorted(e.keys())
    l = list()
    for key in c:
        value = e[key]
        if value is not None and type(value) == str:
            for old in ["!", "'", "(", ")", "*"]:
                value = value.replace(old, '')
            l.append(encodeURIComponent(key) + "=" + encodeURIComponent(value))
    v = "&".join(l)
    return md5((v + a).encode()).hexdigest(), str(u)


from .article import get_article
from .video import get_video, get_video_dm, get_video_comments
import logging

logging.basicConfig(
    level=logging.DEBUG,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
    datefmt="%Y-%m-%d %H:%M:%S"
)

__all__ = ['get_article', 'get_video', 'get_video_dm', 'get_video_comments']
